
from django.core.exceptions import PermissionDenied

from rolepermissions.roles import RolesManager
from rolepermissions.verifications import has_role, has_permission


def has_role_decorator(role):
    def request_decorator(dispatch):
        def wrapper(request, *args, **kwargs):
            user = request.user
            if user.is_authenticated():
                if has_role(user, role):
                    return dispatch(request, *args, **kwargs)

            raise PermissionDenied
        return wrapper
    return request_decorator


def has_permission_decorator(permission_name):
    def request_decorator(dispatch):
        def wrapper(request, *args, **kwargs):
            user = request.user
            if user.is_authenticated():
                if has_permission(user, permission_name):
                    return dispatch(request, *args, **kwargs)

            raise PermissionDenied
        return wrapper
    return request_decorator




from django.test import TestCase
from django.contrib.auth import get_user_model
from django.http import Http404

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole
from rolepermissions.shortcuts import (
    get_user_role, get_user_permissions, grant_permission
)
from rolepermissions.models import UserPermission
from rolepermissions.verifications import has_permission


class Role1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }

class Role2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }

class Role3(AbstractUserRole):
    role_name = 'new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }


class GetUserRoleTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())

    def test_get_user_role(self):
        user = self.user

        user_role = Role1.assign_role_to_user(user)

        self.assertEquals(get_user_role(user), Role1)

    def test_get_user_role_after_role_change(self):
        user = self.user

        user_role = Role1.assign_role_to_user(user)
        user_role = Role3.assign_role_to_user(user)

        self.assertEquals(get_user_role(user), Role3)

    def test_user_without_role(self):
        user = self.user

        self.assertIsNone(get_user_role(user))

    def tearDown(self):
        RolesManager._roles = {}


class GetUserPermissionsTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())
        self.user_role = Role2.assign_role_to_user(self.user)

    def test_get_user_permissinons(self):
        user = self.user

        permissions = get_user_permissions(user)

        self.assertIn('permission3', permissions)
        self.assertIn('permission4', permissions)
        self.assertTrue(permissions['permission3'])
        self.assertFalse(permissions['permission4'])
        self.assertEquals(len(permissions), 2)

    def test_creates_when_does_not_exists(self):
        user = self.user

        class Role4(AbstractUserRole):
            available_permissions = {
                'the_permission': True,
            }
        RolesManager.register_role(Role4)

        Role4.assign_role_to_user(user)

        Role4.available_permissions = {
            'the_permission': True,
            'new_permission': True,
        }

        permissions = get_user_permissions(user)

        self.assertIn('the_permission', permissions)
        self.assertIn('new_permission', permissions)
        self.assertTrue(permissions['the_permission'])
        self.assertTrue(permissions['new_permission'])
        self.assertEquals(len(permissions), 2)

        new_permission = UserPermission.objects.get(user=user, permission_name='new_permission')

        self.assertTrue(new_permission.is_granted)


class GrantPermissionTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())
        self.user_role = Role2.assign_role_to_user(self.user)

    def test_grant_permission(self):
        user = self.user
        
        self.assertTrue(grant_permission(user, 'permission4'))

        self.assertTrue(has_permission(user, 'permission4'))

    def test_grat_granted_permission(self):
        user = self.user
        
        self.assertTrue(grant_permission(user, 'permission3'))

        self.assertTrue(has_permission(user, 'permission3'))

    def test_not_allowed_permission(self):
        user = self.user

        self.assertFalse(grant_permission(user, 'permission1'))




from django.test import TestCase
from django.contrib.auth import get_user_model

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole
from rolepermissions.models import UserPermission

class Role1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }

class Role2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }

class Role3(AbstractUserRole):
    role_name = 'new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }


class AbstractUserRoleTests(TestCase):

    def setUp(self):
        pass

    def test_get_name(self):
        self.assertEquals(Role1.get_name(), 'role1')
        self.assertEquals(Role2.get_name(), 'role2')
        self.assertEquals(Role3.get_name(), 'new_name')

    def test_assign_Role1_default_permissions(self):
        user = mommy.make(get_user_model())

        Role1.assign_default_permissions(user)
        permissions = UserPermission.objects.filter(user=user)

        permission_hash = { p.permission_name: p.is_granted for p in permissions }

        self.assertIn('permission1', permission_hash)
        self.assertTrue(permission_hash['permission1'])
        self.assertIn('permission2', permission_hash)
        self.assertTrue(permission_hash['permission2'])
        self.assertEquals(len(permissions), 2)

    def test_assign_Role2_default_permissions(self):
        user = mommy.make(get_user_model())

        Role2.assign_default_permissions(user)
        permissions = UserPermission.objects.filter(user=user)

        permission_hash = { p.permission_name: p.is_granted for p in permissions }

        self.assertIn('permission3', permission_hash)
        self.assertTrue(permission_hash['permission3'])
        self.assertIn('permission4', permission_hash)
        self.assertFalse(permission_hash['permission4'])
        self.assertEquals(len(permissions), 2)

    def test_assign_Role3_default_permissions(self):
        user = mommy.make(get_user_model())

        Role3.assign_default_permissions(user)
        permissions = UserPermission.objects.filter(user=user)

        permission_hash = { p.permission_name: p.is_granted for p in permissions }

        self.assertIn('permission5', permission_hash)
        self.assertFalse(permission_hash['permission5'])
        self.assertIn('permission6', permission_hash)
        self.assertFalse(permission_hash['permission6'])
        self.assertEquals(len(permissions), 2)

    def test_assign_role_to_user(self):
        user = mommy.make(get_user_model())

        user_role = Role1.assign_role_to_user(user)

        self.assertEquals(user_role.role_name, 'role1')

    def test_instanciate_role(self):
        user = mommy.make(get_user_model())

        user_role = Role1.assign_role_to_user(user)

        self.assertIsNotNone(user_role.pk)

    def test_change_user_role(self):
        user = mommy.make(get_user_model())

        user_role = Role1.assign_role_to_user(user)

        self.assertEquals(user_role.role_name, 'role1')

        user_role = Role2.assign_role_to_user(user)

        self.assertEquals(user_role.role_name, 'role2')

    def test_delete_old_permissions_on_role_change(self):
        user = mommy.make(get_user_model())

        Role1().assign_role_to_user(user)
        
        permissions = UserPermission.objects.filter(user=user)

        permission_names = [n.permission_name for n in permissions]

        self.assertIn('permission1', permission_names)
        self.assertIn('permission2', permission_names)
        self.assertEquals(len(permissions), 2)

        Role2.assign_role_to_user(user)

        permissions = UserPermission.objects.filter(user=user)

        permission_names = [n.permission_name for n in permissions]

        self.assertIn('permission3', permission_names)
        self.assertIn('permission4', permission_names)
        self.assertEquals(len(permissions), 2)


    def test_permission_list(self):
        self.assertIn('permission1', Role1.permission_list())
        self.assertIn('permission2', Role1.permission_list())

        self.assertIn('permission3', Role2.permission_list())
        self.assertIn('permission4', Role2.permission_list())


class RolesManagerTests(TestCase):

    def setUp(self):
        RolesManager._roles = {}

    def test_registering_role(self):
        self.assertNotIn('role1', RolesManager._roles)

        RolesManager.register_role(Role1)

        self.assertIn('role1', RolesManager._roles)

    def test_get_roles(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)

        self.assertTrue(isinstance(RolesManager.get_roles(), dict))
        self.assertEquals(len(RolesManager.get_roles()), 2)

    def test_retrieve_role(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)

        self.assertEquals(RolesManager.retrieve_role('role1'), Role1)
        self.assertEquals(RolesManager.retrieve_role('role2'), Role2)




import inspect
import re

from django.core.exceptions import ObjectDoesNotExist

from rolepermissions.models import UserRole, UserPermission


def camelToSnake(s):
    """ 
    https://gist.github.com/jaytaylor/3660565
    Is it ironic that this function is written in camel case, yet it
    converts to snake case? hmm..
    """
    _underscorer1 = re.compile(r'(.)([A-Z][a-z]+)')
    _underscorer2 = re.compile('([a-z0-9])([A-Z])')

    subbed = _underscorer1.sub(r'\1_\2', s)
    return _underscorer2.sub(r'\1_\2', subbed).lower()


class RolesManager(object):
    _roles = {}

    @classmethod
    def register_role(cls, role):
        cls._roles[role().get_name()] = role

    @classmethod
    def get_roles(cls):
        return cls._roles

    @classmethod
    def retrieve_role(cls, role_name):
        if role_name in cls._roles:
            return cls._roles[role_name]


class AbstractUserRole(object):

    @classmethod
    def get_name(cls):
        if hasattr(cls, 'role_name'):
            return cls.role_name

        return camelToSnake(cls.__name__)

    @classmethod
    def assign_role_to_user(cls, user):
        try:
            user_role = UserRole.objects.get(user=user)
        except ObjectDoesNotExist:
            user_role = None

        UserPermission.objects.filter(user=user).delete()

        if not user_role:
            user_role = UserRole(user=user)

        user_role.role_name = cls.get_name()
        user_role.save()

        user.role = user_role

        cls.assign_default_permissions(user)

        return user_role

    @classmethod
    def assign_default_permissions(cls, user):
        role_permissions = cls.available_permissions
        for permission in role_permissions:
            UserPermission.objects.get_or_create(
                user=user, permission_name=permission,
                is_granted=role_permissions[permission])

    @classmethod
    def permission_list(cls):
        return cls.available_permissions.keys()

    @classmethod
    def get_default(cls, permission_name):
        return cls.available_permissions[permission_name]





from django.http import Http404

from rolepermissions.roles import RolesManager
from rolepermissions.models import UserPermission

def get_user_role(user):
    if hasattr(user, 'role'):
        return RolesManager.retrieve_role(user.role.role_name)
    return None


def get_user_permissions(user):
    role = get_user_role(user)

    permissions = UserPermission.objects.filter(user=user)
    permissions = { p.permission_name: p for p in permissions }

    user_permissions = []
    for permission_name in role.permission_list():
        if permission_name in permissions:
            permission = permissions[permission_name]
        else:
            permission = UserPermission(user=user, 
                permission_name=permission_name, 
                is_granted=role.get_default(permission_name))
            permission.save()
        user_permissions.append(permission)

    permission_hash = { p.permission_name: p.is_granted for p in user_permissions }

    return permission_hash


def grant_permission(user, permission_name):
    user_permissions = get_user_permissions(user)

    if permission_name in user_permissions:
        permission = UserPermission.objects.get(user=user, 
            permission_name=permission_name)

        permission.is_granted = True
        permission.save()

        return True

    return False




from rolepermissions.exceptions import CheckerNotRegistered

class PermissionsManager(object):
    _checkers = {}

    @classmethod
    def register_checker(cls, name, function):
        cls._checkers[name] = function

    @classmethod
    def get_checkers(cls):
        return cls._checkers

    @classmethod
    def retrieve_checker(cls, checker_name):
        if checker_name in cls._checkers:
            return cls._checkers[checker_name]

        raise CheckerNotRegistered('Checker with name %s was not registered' % checker_name)


def register_object_checker(name=None):
    def fuction_decorator(func):
        checker_name = name if name else func.__name__
        PermissionsManager.register_checker(checker_name, func)
    return fuction_decorator




from django.views.generic import DetailView
from django.utils.decorators import method_decorator
from django.test import TestCase
from django.contrib.auth import get_user_model, login
from django.test.client import RequestFactory
from django.core.exceptions import PermissionDenied

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole
from rolepermissions.mixins import HasRoleMixin, HasPermissionsMixin


class Role1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }


class Role2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }


class HasRoleDetailView(HasRoleMixin, DetailView):
    allowed_roles = ['role1']

    def get_object(self):
        return True

class MultipleHasRoleDetailView(HasRoleMixin, DetailView):
    allowed_roles = ['role1', Role2]

    def get_object(self):
        return True

class HasRoleDecoratorTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)

        self.user = mommy.make(get_user_model())

        self.factory = RequestFactory()

        self.request = self.factory.get('/')
        self.request.session = {}
        self.request.user = self.user

    def test_has_allowed_role_to_view(self):
        user = self.user
        request = self.request

        Role1.assign_role_to_user(user)

        response = HasRoleDetailView.as_view()(request)

        self.assertEquals(response.status_code, 200)

    def test_does_not_have_allowed_role_to_view(self):
        user = self.user
        request = self.request

        Role2.assign_role_to_user(user)

        with self.assertRaises(PermissionDenied):
            response = HasRoleDetailView.as_view()(request)

    def test_view_with_multiple_allowed_roles(self):
        user = self.user
        request = self.request

        Role2.assign_role_to_user(user)

        response = MultipleHasRoleDetailView.as_view()(request)

        self.assertEquals(response.status_code, 200)

        Role1.assign_role_to_user(user)

        response = MultipleHasRoleDetailView.as_view()(request)

        self.assertEquals(response.status_code, 200)

    def tearDown(self):
        RolesManager._roles = {}


class HasPermissionDetailView(HasPermissionsMixin, DetailView):
    required_permission = 'permission2'

    def get_object(self):
        return True


class HasPermissionDecoratorTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)

        self.user = mommy.make(get_user_model())

        self.factory = RequestFactory()

        self.request = self.factory.get('/')
        self.request.session = {}
        self.request.user = self.user

    def test_has_permission_granted(self):
        user = self.user
        request = self.request

        Role1.assign_role_to_user(user)

        response = HasPermissionDetailView.as_view()(request)

        self.assertEquals(response.status_code, 200)

    def test_permission_denied(self):
        user = self.user
        request = self.request

        Role2.assign_role_to_user(user)

        with self.assertRaises(PermissionDenied):
            response = HasPermissionDetailView.as_view()(request)

    def tearDown(self):
        RolesManager._roles = {}




from django.test import TestCase
from django.contrib.auth import get_user_model

from model_mommy import mommy

from rolepermissions.roles import RolesManager, AbstractUserRole
from rolepermissions.verifications import has_role, has_permission, has_object_permission
from rolepermissions.permissions import PermissionsManager, register_object_checker
from rolepermissions.models import UserPermission


class Role1(AbstractUserRole):
    available_permissions = {
        'permission1': True,
        'permission2': True,
    }

class Role2(AbstractUserRole):
    available_permissions = {
        'permission3': True,
        'permission4': False,
    }

class Role3(AbstractUserRole):
    role_name = 'new_name'
    available_permissions = {
        'permission5': False,
        'permission6': False,
    }


class TestHasRoleTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())

        Role1.assign_role_to_user(self.user)

    def test_user_has_Role1(self):
        user = self.user

        self.assertTrue(has_role(user, Role1))

    def test_user_does_not_have_Role2(self):
        user = self.user

        self.assertFalse(has_role(user, Role2))

    def test_user_has_Role1_or_Role2(self):
        user = self.user

        self.assertTrue(has_role(user, [Role1, Role2]))

    def test_has_role_by_name(self):
        user = self.user

        self.assertTrue(has_role(user, 'role1'))

    def test_user_has_Role1_or_Role3_by_name(self):
        user = self.user

        Role3.assign_role_to_user(user)

        self.assertTrue(has_role(user, ['role1', 'new_name']))

    def test_not_existent_role(self):
        user = self.user

        self.assertFalse(has_role(user, 'not_a_role'))


class HasPermissionTests(TestCase):
    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())

        Role1.assign_role_to_user(self.user)

    def test_has_Role1_permission(self):
        user = self.user

        self.assertTrue(has_permission(user, 'permission1'))

    def test_dos_not_have_Role1_permission(self):
        user = self.user

        Role1.assign_role_to_user(user)

        self.assertFalse(has_permission(user, 'permission3'))

    def test_creates_permission_when_not_existent(self):
        user = self.user

        class Role4(AbstractUserRole):
            available_permissions = {
                'the_permission': True
            }
        RolesManager.register_role(Role4)

        Role4.assign_role_to_user(user)

        Role4.available_permissions = { 'different_one': True }

        self.assertTrue(has_permission(user, 'different_one'))

        permission = UserPermission.objects.get(user=user, permission_name='different_one')

        self.assertTrue(permission.is_granted)

    def test_does_not_creates_if_permission_does_not_exists_in_role(self):
        user = self.user

        Role1.assign_role_to_user(user)

        self.assertFalse(has_permission(user, 'different_permission'))

        try:
            permission = UserPermission.objects.get(user=user, 
                permission_name='different_permission')
        except:
            permission = None

        self.assertIsNone(permission)

    def test_not_existent_permission(self):
        user = self.user

        self.assertFalse(has_permission(user, 'not_a_permission'))


class HasObjectPermissionTests(TestCase):

    def setUp(self):
        RolesManager.register_role(Role1)
        RolesManager.register_role(Role2)
        RolesManager.register_role(Role3)

        self.user = mommy.make(get_user_model())

        Role1.assign_role_to_user(self.user)

        @register_object_checker()
        def obj_checker(role, user, obj):
            return obj and True

    def test_has_object_permission(self):
        user = self.user

        self.assertTrue(has_object_permission('obj_checker', user, True))

    def test_does_not_have_object_permission(self):
        user = self.user

        self.assertFalse(has_object_permission('obj_checker', user, False))




from django.db import models
from django.contrib.auth import get_user_model


class UserRole(models.Model):
    user = models.OneToOneField(get_user_model(), related_name='role')
    role_name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.user.get_full_name() + ' - ' + self.role_name


class UserPermission(models.Model):
    user = models.ForeignKey(get_user_model(), related_name='+')
    permission_name = models.CharField(max_length=255)
    is_granted = models.BooleanField(default=False)

    def __unicode__(self):
        return self.user.get_full_name() + ' - ' + self.permission_name + ' - ' + str(self.is_granted)




import inspect

from django.conf import settings
from django.utils.importlib import import_module

from rolepermissions.roles import AbstractUserRole, RolesManager
from rolepermissions.permissions import PermissionsManager


def load_roles_and_permissions():
    roles_module = import_module(settings.ROLEPERMISSIONS_MODULE)

    RolesManager._roles = {}
    PermissionsManager._checkers = {}

    for name, cls in roles_module.__dict__.items():
        if inspect.isclass(cls) and issubclass(cls, AbstractUserRole) and cls is not AbstractUserRole:
            RolesManager.register_role(cls)
            # print 'Loading role ' + cls().get_name()

    for app_name in settings.INSTALLED_APPS:
        if app_name is not 'rolepermissions':
            try:
                import_module('.permissions', app_name)
            except ImportError as exc:
                pass

